/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package com.droidlar;

import com.rapplogic.xbee.api.ApiId;
import com.rapplogic.xbee.api.PacketListener;
import org.apache.log4j.PropertyConfigurator;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeAddress16;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.zigbee.ZNetRxResponse;
import com.rapplogic.xbee.api.zigbee.ZNetTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetTxStatusResponse;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Classe responsável pelo gerenciamento da comunicação com os módulos xBee.
 */
public class GererenciadorDosModulos implements PacketListener{

    private Vector<XBeeAddress64> enderecos;
    private Vector<String[]> nomes;
    private Vector<String[]> tipos;
    private Vector<String[]> estados;

    private Vector<String> endBackup;
    private Vector<String[]> nomesBuckup;

    private Servidor root;
    private ThreadVarredura tVarredura;
    private XBee xbee;

    private final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(getClass());

    public static final String REQUEST_INFO = "8";
    public static final int RESPONSE_INFO = 9;
    public static final int UPDATE_ESTADO = 10;

    public GererenciadorDosModulos(Servidor pai) {

        PropertyConfigurator.configure(getClass().getResource("log4j.properties"));

        root = pai;
        // Objeto que permite a manipulação do modulo xBee
        xbee = new XBee();

        nomes = new Vector<String[]>();
        enderecos = new Vector<XBeeAddress64>();
        tipos = new Vector<String[]>();
        estados = new Vector<String[]>();

        // Restaura lista com os nomes dos dispositivos alterados pelo usuário
        nomesBuckup = (Vector<String[]>)root.recuperarObjeto("nomesBackup.obj");

        if(nomesBuckup == null){
            nomesBuckup = new Vector<String[]>();
        }

        // Restaura lista com os endereços dos dispositivos que tiveram seus nomes
        // alterados pelo usuário
        endBackup = (Vector<String>)root.recuperarObjeto("endBackup.obj");

        if(endBackup == null){
            endBackup = new Vector<String>();
        }

        tVarredura = new ThreadVarredura(this);

        conectaModulo();
    }

    /**
     * Abre conexão na porta serial onde o modulo xBee coordenador está conectado.
     */
    public void conectaModulo(){
        try {

            String serial = "/dev/ttyUSB0";
            // abre conexão na porta COM do coodenador a um baud rate específico
            xbee.open(serial, 9600);

            // configura o listener para o recebimento de pacotes
            // essa linha tem que vir depois do "xbee.open()"
            xbee.addPacketListener(this);

        } catch (Exception ex) {
            Logger.getLogger(GererenciadorDosModulos.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Envia mensagem ao módulo xBee
     * @param tipo tipo de mensagem enviada ao módulo
     * @param mensagem mensagem enviada ao módulo
     * @param opcao campo adicional que pode ser utilizado para indicar para qual
     * dispositivo a mensagem é destinada caso o módulo controle mais de um dispositivo
     * @param modulo endereço do módulo
     * @return true se a mensagem foi enviada corretamente
     */
    public boolean enviarDados(String tipo, String mensagem, String opcao, String modulo){    
        XBeeAddress64 endereco = root.convertStringToX64(modulo);
        return enviar(tipo, mensagem, opcao, endereco);
    }

    /**
     * Envia mensagem ao módulo xBee
     * @param tipo tipo de mensagem enviada ao módulo
     * @param mensagem mensagem enviada ao módulo
     * @param opcao campo adicional que pode ser utilizado para indicar para qual
     * dispositivo a mensagem é destinada caso o módulo controle mais de um dispositivo
     * @param modulo endereço do módulo
     * @return true se a mensagem foi enviada corretamente
     */
    public boolean enviarDados(String tipo, String mensagem, String opcao, XBeeAddress64 modulo){
        return enviar(tipo, mensagem, opcao, modulo);
    }

    private boolean enviar(String tipo, String mensagem, String opcao, XBeeAddress64 modulo){

        if ((modulo != null) && xbee.isConnected()) {
            // create an array of arbitrary data to send
            int[] payload = new int[]{Integer.parseInt(tipo), Integer.parseInt(mensagem), Integer.parseInt(opcao)};
            // first request we just send 64-bit address.  we get 16-bit network address with status response
            ZNetTxRequest request = new ZNetTxRequest(modulo, payload);

            try {

                XBeeResponse xResponse = xbee.sendSynchronous(request, 10000);

                if (xResponse instanceof ZNetTxStatusResponse) {

                    ZNetTxStatusResponse response = (ZNetTxStatusResponse) xResponse;
                    // update frame id for next request
                    request.setFrameId(xbee.getNextFrameId());

                    if (response.getDeliveryStatus() == ZNetTxStatusResponse.DeliveryStatus.SUCCESS) {
                        // the packet was successfully delivered
                        if (response.getRemoteAddress16().equals(XBeeAddress16.ZNET_BROADCAST)) {
                            // specify 16-bit address for faster routing?.. really only need to do this when it changes
                            request.setDestAddr16(response.getRemoteAddress16());
                        }

                        if (tipo.equals("" + Servidor.MSG_CONTROLE)) {
                            int index = enderecos.indexOf(modulo);

                            if (index != -1) {

                                String[] temp = estados.get(index);
                                temp[Integer.parseInt(opcao)] = mensagem;
                                estados.set(index, temp);

                            }
                        }
                    } else {
                        // packet failed.  log error
                        // it's easy to create this error by unplugging/powering off your remote xbee.  when doing so I get: packet failed due to error: ADDRESS_NOT_FOUND
                        log.error("packet failed due to error: " + response.getDeliveryStatus());

                        return false;
                    }
                }

            } catch (Exception e) {

                log.warn("request timed out");
                Logger.getLogger(GererenciadorDosModulos.class.getName()).log(Level.SEVERE, null, e);
                return false;
            }
        }else{
            return false;
        }
        
        return true;
    }

    @Override
    public void processResponse(XBeeResponse response) {

         // verifica se a mensagem é originada de um outro módulo
         if (response.getApiId() == ApiId.ZNET_RX_RESPONSE) {

            // Convertendo o pacote recebido
            ZNetRxResponse rx = (ZNetRxResponse) response;

            int[] dados = rx.getData();

            switch (dados[0]) {

                // mensagem recebida quando o estado do dispositivo é alterado manualmente
                case UPDATE_ESTADO:

                    System.out.println("\nTipo de resposta: " + dados[0]);
                    System.out.println("\nEstado: " + dados[1]);
                    System.out.println("\nLed: " + dados[2]);
                    System.out.println("\nModulo: " + rx.getRemoteAddress64());

                    // procura pelo endereço do módulo na lista de endereços,
                    // caso encontre então substitui seu estado na lista de estados
                    // com o valor recebido
                    if(enderecos.size() > 0){
                        int index = enderecos.indexOf(rx.getRemoteAddress64());

                        if (index != -1) {
                            String[] temp = estados.get(index);
                            temp[dados[2]] = "" + dados[1];
                            estados.set(index, temp);
                        }
                    }

                    break;

                // mensagem recebida quando o módulo é inicializado ou
                // quando ele responde a requisição de varredura
                case RESPONSE_INFO:

                    XBeeAddress64 address = rx.getRemoteAddress64();

                    int dispQtd = (dados.length-1)/2;
                    
                    String[] vetorTipo = new String[dispQtd];
                    String[] vetorEst = new String[dispQtd];

                    for(int i=0; i<dispQtd; i++){
                        vetorTipo[i] = "" + dados[i+1];
                        vetorEst[i] = "" + dados[i+1+dispQtd];
                    }

                    // procura pelo endereço do módulo na lista de endereços,
                    // caso não encontre então adiciona as informações recebidas
                    if (!enderecos.contains(address)) {

                        enderecos.add(address);
                        tipos.add(vetorTipo);
                        estados.add(vetorEst);

                        int index = endBackup.indexOf(root.convertX64ToString(address));

                        // Verifica se dispositivo encontrado já estava na lista
                        // de backup de nomes, caso esteja, adiciona o nome da lista de backup
                        // em vez do nome recebido
                        if (index != -1) {
                            nomes.add(nomesBuckup.get(index));
                        } else {
                            // Se o dispositivo não estava na lista então configura
                            // ele com o nome padão

                            String[] defaultNomes = new String[dispQtd];
                            for (int i = 0; i < dispQtd; i++) {
                                defaultNomes[i] = "Sem nome";
                            }
                            nomes.add(defaultNomes);
                            nomesBuckup.add(defaultNomes);
                            endBackup.add(root.convertX64ToString(address));

                            // salva estes objetos em disco
                            root.gravarObjeto(nomesBuckup, "nomesBackup.obj");
                            root.gravarObjeto(endBackup, "endBackup.obj");
                        }
                    } else {
                        // caso encontre o endereço do módulo na lista de endereços
                        // então substitui o estado na lista pelo valor recebido

                        int index = enderecos.indexOf(address);

                        if (index != -1) {
                            estados.set(index, vetorEst);
                        }

                    }

                    break;
           
            }

        } 
    }

    /**
     * Ativa ou desativa a thread que faz a varredura periodicamente.
     * @param dados contem informações como o intervalo de tempo entre as varreduras
     */
    public void gerVarredura(final String dados){

        new Thread() {
            public void run() {

                if (!dados.equals("")) {

                    String[] valores = dados.split(":");
                    boolean ativa;

                    if (Integer.parseInt(valores[0]) == 1) {
                        ativa = true;
                    } else {
                        ativa = false;
                    }

                    if (ativa == true) {
                        if (tVarredura.isAlive()) {
                            tVarredura.paraThread();
                        }

                        tVarredura = new ThreadVarredura(GererenciadorDosModulos.this);

                        long time = Long.parseLong(valores[1]);
                        if (time == 0) {
                            tVarredura.setSleepTime(60 * 1000);
                        } else {
                            tVarredura.setSleepTime(time * 60 * 1000);
                        }
                        
                        tVarredura.start();
                        System.out.println("VARREDURA PERIODICA ATIVADA: " + time + "min");

                    } else if (tVarredura.isAlive()) {
                        tVarredura.paraThread();
                        System.out.println("VARREDURA PERIODICA DESATIVADA");
                    }
                }
            }
        }.start();
    }

    /**
     * Retorna as informações de um determinado tipo de dispositivo.
     * @param tipo código do tipo de dispositivo que se deseja obter as informações
     * @return string contendo lista com informaçẽos dos dipositivos
     */
    public String getPorTipo(String tipo){       
        String retorno = "";
        
        if (tipos.size() > 0) {
            for (int i = 0; i < tipos.size(); i++) {

                String[] t = tipos.get(i);
                for (int k = 0; k < t.length; k++) {
                    if (t[k].equals(tipo)) {

                        if (k == 0) {
                            retorno += "" + root.convertX64ToString(enderecos.get(i)) + "%";
                        }

                        retorno += nomes.get(i)[k] + ":" + k + ":" + estados.get(i)[k] + "%";

                        if (k == t.length-1) {
                            retorno += "#";
                        }
                    }
                }
            }
        }else{
            retorno = "null";
        }

        return retorno;
    }

    /**
     * Retorna as informações de todos os dispositivos.
     * @return string com as informações dos dispositivos
     */
    public String getDisps(){
        String retorno = "";

        if (enderecos.size() > 0) {
            for (int i = 0; i < enderecos.size(); i++) {

                String[] t = tipos.get(i);
                for (int k = 0; k < t.length; k++) {
                    if (k == 0) {
                        retorno += "" + root.convertX64ToString(enderecos.get(i)) + "%";
                    }

                    retorno += nomes.get(i)[k] + ":" + tipos.get(i)[k] + "%";

                    if (k == t.length - 1) {
                        retorno += "#";
                    }
                }
            }
        }else{
            retorno = "null";
        }

        return retorno;
    }

    /**
     * Atualiza a lista de nomes dos dispositivos.
     * @param mensagem string com os nomes e endereços dos dispositivos
     */
    public void setNomeDisp(String mensagem){

        if (!mensagem.equals("")) {

            if (enderecos.size() > 0) {

                String[] partes = mensagem.split("%");

                for (int k = 0; k < partes.length; k++) {

                    String[] item = partes[k].split("#", 2);
                    String end = item[0];

                    XBeeAddress64 address = root.convertStringToX64(end);

                    item = item[1].split("#");

                    int i = 0;
                    boolean encontrou = false;

                    while ((i < enderecos.size()) && (encontrou == false)) {
                        if (enderecos.get(i).equals(address)) {
                            String[] temp = nomes.get(i);

                            for (int n = 0; n < item.length; n++) {
                                if (n < temp.length) {
                                    temp[n] = item[n];
                                }
                            }

                            nomes.set(i, temp);
                            
                            int index = endBackup.indexOf(root.convertX64ToString(address));

                            if (index != -1) {
                                nomesBuckup.set(index, temp);
                                // salva estes objetos em disco
                                root.gravarObjeto(nomesBuckup, "nomesBackup.obj");
                            }

                            encontrou = true;
                        }
                        i++;
                    }
                }
            }
        }
    }

    public ThreadVarredura getThreadVar(){
        return tVarredura;
    }

    /**
     * Retorna lista com os endereços dos módulos xBee.
     * @return lista com os endereços
     */
    public Vector<XBeeAddress64> getEnderecos() {
        return enderecos;
    }

    /**
     * Retorna lista com os nomes dos módulos xBee.
     * @return lista com os nomes
     */
    public Vector<String[]> getNomes() {
        return nomes;
    }

    /**
     * Retorna lista com os tipos dos módulos xBee.
     * @return lista com os tipos
     */
    public Vector<String[]> getTipos() {
        return tipos;
    }

    /**
     * Retorna lista com os estados dos módulos xBee.
     * @return lista com os estados
     */
    public Vector<String[]> getEstados() {
        return estados;
    }

    /**
     * Retorna objeto que manipula o módulo xBee.
     * @return objeto de manipulação
     */
    public XBee getXbee() {
        return xbee;
    }

    /**
     * Configura lista de endereços dos módulos xBee
     * @param end lista de endereços
     */
    public void setEnderecos(Vector<XBeeAddress64> end) {
        this.enderecos = end;
    }

    /**
     * Configura lista de nomes dos dispositivos
     * @param end lista de nomes
     */
    public void setNomes(Vector<String[]> n) {
        this.nomes = n;
    }

    /**
     * Configura lista de tipos dos dispositivos
     * @param end lista de tipos
     */
    public void setTipos(Vector<String[]> t) {
        this.tipos = t;
    }

    /**
     * Configura lista de estados dos dispositivos
     * @param end lista de estados
     */
    public void setEstados(Vector<String[]> e) {
        this.estados = e;
    }

}
