/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package com.droidlar;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Classe que gerencia as informações dos perfis.
 */
public class Infos implements Serializable{

    private String uuid;
    private ArrayList<InfoDisp> infoDisp;
    private transient ArrayList<AgendadorDePerfis> agendados;

    public Infos() {
        uuid = "nulo";
        infoDisp = new ArrayList<InfoDisp>();
    }

    /**
     * Gera as listas com as informações dos perfis.
     * @param root objeto para acessar os métodos da classe Servidor
     * @param info string contendo as informações dos perfis
     */
    public void gerarInfos(Servidor root, String info) {

        String[] partes = info.split("%", 2);

        // se UUID mudar significa que houveram alteracoes nos perfis
        if (!partes[0].equals(uuid)) {

            // encerra perfis programados anteriormente
            mataPerfis();

            // limpa lista atual e atualiza UUID
            infoDisp.clear();
            uuid = partes[0];

            if (partes.length > 1) {

                // remove primeira posicao do vetor (UUID)
                partes = partes[1].split("%");

                for (String linha : partes) {

                    String[] listaTemp = linha.split("#", 2);

                    // pega endereco do dispositivo
                    String endereco = listaTemp[0];
                    // ex de conteudo: Teste;Sem nome;00:00;04:00;1;79:0;1:2:3:4:5:6:7
                    String[] dispLista = listaTemp[1].split("#");

                    for (String dispItem : dispLista) {

                        String[] itens = dispItem.split(";", 7);

                        String pNome = itens[0];
                        String dNome = itens[1];
                        String[] ini = itens[2].split(":");
                        String[] fim = itens[3].split(":");
                        String dispID = itens[4];
                        String[] valores = itens[5].split(":");

                        //cria vetor com valor dos dias
                        String[] listaDia = itens[6].split(":");

                        int[] intDia = new int[listaDia.length];
                        for (int n = 0; n < listaDia.length; n++) {
                            intDia[n] = Integer.parseInt(listaDia[n]);
                        }

                        InfoDisp dadosTemp = new InfoDisp();

                        dadosTemp.perfNome = pNome;
                        dadosTemp.dispNome = dNome;
                        dadosTemp.endereco = endereco;
                        dadosTemp.ID = dispID;
                        dadosTemp.ini_hora = Integer.parseInt(ini[0]);
                        dadosTemp.ini_min = Integer.parseInt(ini[1]);
                        dadosTemp.fim_hora = Integer.parseInt(fim[0]);
                        dadosTemp.fim_min = Integer.parseInt(fim[1]);
                        dadosTemp.valorInicial = valores[0];
                        dadosTemp.valorFinal = valores[1];
                        dadosTemp.dias = intDia;

                        infoDisp.add(dadosTemp);
                    }
                }
                this.agendaTarefas(root);
            }
            // salva este objeto em disco
            root.gravarObjeto(this, "perfis.obj");
        }
    }

    /**
     * Agenda as tarefas dos perfis.
     * @param root objeto para acessar os métodos da classe Servidor
     */
    public void agendaTarefas(Servidor root) {

        agendados = new ArrayList<AgendadorDePerfis>();

        if (infoDisp.size() > 0) {
            for (InfoDisp inf : infoDisp) {

                System.out.println("\nID: " + inf.ID);
                System.out.println("Endereco: " + inf.endereco);
                System.out.println("Horario Inicial: " + inf.ini_hora + ":" + inf.ini_min);
                System.out.println("Valor Inicial: " + inf.valorInicial);
                System.out.println("Horario Final: " + inf.fim_hora + ":" + inf.fim_min);
                System.out.println("Valor Final: " + inf.valorFinal);

                String dias = "";
                for (int i : inf.dias) {
                    dias += " " + i;
                }
                System.out.println("Dias da semana:" + dias);

                agendados.add(new AgendadorDePerfis(root, inf));
            }
        }
    }

    /**
     * Finaliza perfis agendados.
     */
    private void mataPerfis(){
        if (agendados != null) {
            if (agendados.size() > 0) {
                for (AgendadorDePerfis ap : agendados) {
                    ap.mataPerfil();
                }
            }
        }
    }

    /**
     * Retorna informações dos perfis agendados.
     * @return string com informações dos perfis agendados
     */
    public String getPerfis(){

        String info = "";

        String addrAnterior = "";

        for (int i = 0; i < infoDisp.size(); i++) {

            String address = infoDisp.get(i).endereco;

            if(!address.equals(addrAnterior)){
              info += "%"+address;
            }

            info += "#"+infoDisp.get(i).perfNome+";"+infoDisp.get(i).dispNome+";"+infoDisp.get(i).ini_hora+":"+infoDisp.get(i).ini_min+";"
                 +infoDisp.get(i).fim_hora+":"+infoDisp.get(i).fim_min+";"+infoDisp.get(i).ID+";"
                 +infoDisp.get(i).valorInicial+":"+infoDisp.get(i).valorFinal+";"+getString(infoDisp.get(i).dias);

            addrAnterior = address;
        }
        // evita o erro do metodo substring no retorno
        if(info.length() == 0){
            return "nulo";
        }

        return info.substring(1);
    }

    /**
     * Converte o vetor de inteiros em uma string com os valores em sequencia separados pelo caracter ':'.
     * @param dias vetor com os dias
     * @return string com os valores dos dias
     */
    public String getString(int[] dias){
        String retorno = "";

        for (int i = 0; i < dias.length; i++) {
            retorno += ":"+dias[i];
        }

        return retorno.substring(1);
    }

    /**
     * Retorna o UUID da lista de perfis.
     * @return o valor do UUID
     */
    public String getUUID(){
        return uuid;
    }
}
