/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package com.droidlar;

import com.rapplogic.xbee.api.XBeeAddress64;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Classe que faz requisição a todos os módulos existentes na rede.
 */
public class ThreadVarredura extends Thread{   

    private long sleepTime = 5*1000*60;
    private GererenciadorDosModulos parent;
    private volatile boolean continua;

    public ThreadVarredura(GererenciadorDosModulos pai){

        parent = pai;
    }

    @Override
    public void run(){

        continua = true;

        while(continua){

            verredura();
            
            try {
                Thread.sleep(sleepTime);
            } catch (InterruptedException ex) {
                Logger.getLogger(ThreadVarredura.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Envia uma requisição das informações aos módulos da rede.
     */
    public synchronized void verredura() {

        parent.getEnderecos().clear();
        parent.getNomes().clear();
        parent.getTipos().clear();
        parent.getEstados().clear();

        parent.enviarDados(GererenciadorDosModulos.REQUEST_INFO, "0", "0", XBeeAddress64.BROADCAST);

    }

    /**
     * Finaliza a thread de varredura.
     */
    public void paraThread(){
//        try {
            continua = false;
            interrupt();
//            this.join();
//        } catch (InterruptedException ex) {
//            Logger.getLogger(ThreadVarredura.class.getName()).log(Level.SEVERE, null, ex);
//        }
    }

    /**
     * Configura o intervalo de tempo entre as varreduras executadas pela thread.
     * @param tempo intervalo de tempo entre as varreduras
     */
    public void setSleepTime(long tempo){
        sleepTime = tempo;
    }

}
